/* SStack.cc
 
  SStack adj=0 ox=0 dx=.025 nx=20 < infile.H > outfile.H 

  Applies Slant stack modeling operator to dataset, models a range
  of offsets.  offset = ox + i*dx, where i (int) varies from 0 to nx-1.


  SStack adj=1 os=0 ds=.05 ns=20 < infile.H > outfile.H 

  Applies Adjoint Slant stack operator to (x,t)dataset, output is data in
  tau-p domain. slowness = so + i*ds, where i (int) varies from 0 to ns-1.


  SStack adj=0 inv=1 os=0 ds=.05 ns=20 < infile.H > outfile.H pre=1 rho=1

  Applies Inverse Slant stack operator to (x,t)dataset, output is data in
  tau-p domain. slowness = so + i*ds, where i (int) varies from 0 to ns-1.
  pre=1 -> use diag preconditioner
  rho=1 -> use half rho preconditioner
  */
 
#include <SEPinit.h>
#include <fopsstack.h>
#include <fopdiagscale.h>
#include <fopcontrunc.h>
#include <fopchain.h>
#include <Processnew.h>
#include <floatspace.h>
#include <hestsolver.h>


floatArray rhocoeff = "{{-0.045241},{-0.043407},{-0.068581},{-0.06868},{-0.27409},{ 1},{-0.27409},{-0.06868},{-0.068581},{-0.043407},{-0.045241}}";


main(int argc,char **argv) {
    
	// Seplib initialization
    SepInput * sepInput = SEPinit(argc,argv,SOURCE);

	// Check that we are getting floating point data
    if( sepInput->Info()->Esize() !=4 ) seperr("esize !=4\n");

	// command line arguments
    int adj=0; sepInput->Getch("adj",adj);
    int inv=0; sepInput->Getch("inv",inv);
    int niter=10; sepInput->Getch("niter",niter);
    int pre=0; sepInput->Getch("pre",pre);
    int rho=0; sepInput->Getch("rho",rho);

    Axis x,s,t; // axes to define a slant-stack operator

    floatspace input(sepInput); // make an input space by reading the data 
    Axislist inaxl = input.getaxislist(); // get the axes info

    if( ( adj == 0 && inv==0 ) || ( adj==1 && inv == 1 ) ){
	// going from tau-p to t-x
    	int nx=20; sepInput->Getch("nx",nx);
    	float dx=.025; sepInput->Getch("dx",dx);
    	float ox=.025; sepInput->Getch("ox",ox);
	Axis newx( ox,dx,nx,"Offset");
	// assign to slowness, time and offset axes
	x = newx; t = inaxl["Time"]; s = inaxl["Slowness"];

    }else{
	// going from t-x to tau-p 
       	int ns=20; sepInput->Getch("ns",ns);
       	float ds=.05; sepInput->Getch("ds",ds);
       	float os=0.; sepInput->Getch("os",os);
	Axis news(os,ds,ns,"Slowness");
	// assign to slowness, time and offset axes
	s = news; t = inaxl["Time"]; x = inaxl["Offset"];
    }
 
    fopSStack op(t, x, s); // make SStack operator

    foperator *thefop = &op;  // default operator is just slant stack
    foperator *thepre = nil ; // preconditioner is initially nil

    if( rho == 1 && ( adj==1 || inv == 1 ) ){
       // make a half rho filter operator
       fopContrunc * halfrho = new fopContrunc( rhocoeff, 5, 10, "Time" );
       thefop = new fopChain( &op, halfrho );
       thepre = halfrho;
    }


    if( pre == 1 && ( adj==1 || inv == 1 ) ){
       // make a diagonal preconditioner
       floatspace ones =  input; ones = 1.; 
       floatspace mones =  op.Adjoint( input )(0,0); mones = 1.; 
       floatspace atones = op.Adjoint( ones )(0,0);
	cerr << atones.getarray();
       fopDiagScale *diagpre = new fopDiagScale( mones, atones );

       thefop = new fopChain( thefop, diagpre );
       if( rho == 1 ){
          thepre =  new fopChain( thepre, diagpre );
       }else{ 
          thepre = diagpre; 
       }
    }
    

    floatspacearray output;

     if( inv == 0   ){

        output = Processnew(input, *thefop, adj );

     }else{

	// a solver that reports on stderr
      	HestSolver solv( *thefop, niter ); solv.ReportOn( cerr );
        output = solv.Solve(input);

    }

    // apply preconditioner to get result
    if( thepre != nil && (inv==1 || adj==1) ){
	  output = thepre->Forward( output );
    }


        // write to output
    SepOutput *sepout = new SepOutput("stdout", output(0,0).sepData() );
    delete sepout;

    UnRef(sepInput);
    return 0;
}
